# -*- cperl -*-
# ABSTRACT: class for English localization


use strict;
use warnings;
package SpeL::I18n::en;

use parent 'SpeL::I18n';

# zero derivation

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

SpeL::I18n::en - class for English localization

=head1 VERSION

version 20240616.1738

=head1 SYNOPSYS

Provides text maker for English; this one just uses the parent

=head1 AUTHOR

Walter Daems <wdaems@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2024 by Walter Daems.

This is free software, licensed under:

  The GNU General Public License, Version 3, June 2007

=head1 CONTRIBUTOR

=for stopwords Paul Levrie

Paul Levrie

=cut
